# This is a template Pack __init__. You can use this without changing anything in other packages too!

from .commands import available_commands
from .tables import available_tables

__all__ = ["commands", "tables", "available_commands", "available_tables"]
